#!/bin/sh
#
#	logcheck.sh: Log file checker
#	Written by Craig Rowland <crowland@psionic.com>
#
#	This file needs the program logtail.c to run
#
#	This script checks logs for unusual activity and blatant
#	attempts at hacking. All items are mailed to administrators
# 	for review. This script and the logtail.c program are based upon 
#       the frequentcheck.sh script idea from the Gauntlet(tm) Firewall
#	(c)Trusted Information Systems Inc. The original authors are 
#	Marcus J. Ranum and Fred Avolio.
#
#	Default search files are tuned towards the TIS Firewall toolkit
# 	the TCP Wrapper program. Custom daemons and reporting facilites
#	can be accounted for as well...read the rest of the script for
#	details.
#
#	Version Information
#
#	1.0 	9/29/96  -- Initial Release
#	1.01	11/01/96 -- Added working /tmp directory for symlink protection
#			    (Thanks Richard Bullington (rbulling@obscure.org)
#	1.1	1/03/97	 -- Made this script more portable for Sun's.
#		1/03/97	 -- Made this script work on HPUX
#               5/14/97  -- Added Digital OSF/1 logging support. Big thanks
#                           to Jay Vassos-Libove <libove@compgen.com> for
#                           his changes.
 
# read the config file
. /etc/logcheck/logcheck.conf


umask 077
rm -f $CHECKFILE $CHECKOUTPUT $CHECKREPORT

if [ -f $CHECKFILE -o -f $CHECKOUTPUT -o -f $CHECKREPORT ]; then
	echo "Log files exist in $REALTMPDIR directory that cannot be removed. This may be an attempt to spoof the log checker." \
	| $MAIL -s "$HOSTNAME $DATE ACTIVE SYSTEM ATTACK!" $SYSADMIN
	exit 1
fi

# Check to see if any previous runs have been interrupted and left logs
# lying around
for OLDLOGDIR in $OLDTMPDIRS; do
	(cd $TMPDIR/$OLDLOGDIR
	allow_null_glob_expansion=1
	for OLDCHECK in check.*; do
		cat $OLDCHECK >> $CHECKFILE
		rm -f $OLDCHECK
	done)
	rm -rf $TMPDIR/$OLDLOGDIR
done

# Check the logs specified in the config file.
for lname in $LOGFILES ; do
	[ -r $lname ] && $LOGTAIL $lname >> $CHECKFILE
done

# Set the flag variables
FOUND=0
ATTACK=0

# See if the tmp file exists and actually has data to check, 
# if it doesn't we should erase it and exit as our job is done.
 
if [ ! -s $CHECKFILE ]; then
	rm -f $CHECKFILE
	rm -rf $REALTMPDIR
	exit 0
fi

# Perform Searches

# Check for blatant hacking attempts
if [ -f "$HACKING_FILE" ]; then
	if $GREP -i -f $HACKING_FILE $CHECKFILE > $CHECKOUTPUT; then
		echo >> $CHECKREPORT
		echo "Active System Attack Alerts" >> $CHECKREPORT
		echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=" >> $CHECKREPORT
		cat $CHECKOUTPUT >> $CHECKREPORT
		FOUND=1
		ATTACK=1
	fi
fi

# Check for security violations
if [ -f "$VIOLATIONS_FILE" ]; then
	if $GREP -i -f $VIOLATIONS_FILE $CHECKFILE |
	  $GREP -v -f $VIOLATIONS_IGNORE_FILE > $CHECKOUTPUT; then
		echo >> $CHECKREPORT
		echo "Security Violations" >> $CHECKREPORT
		echo "=-=-=-=-=-=-=-=-=-=" >> $CHECKREPORT
		cat $CHECKOUTPUT >> $CHECKREPORT
		FOUND=1
	fi
fi

# Do reverse grep on patterns we want to ignore
if [ -f "$IGNORE_FILE" ]; then
	if $GREP -v -f $IGNORE_FILE $CHECKFILE > $CHECKOUTPUT; then
		echo >> $CHECKREPORT
		echo "Unusual System Events" >> $CHECKREPORT
		echo "=-=-=-=-=-=-=-=-=-=-=" >> $CHECKREPORT
		cat $CHECKOUTPUT >> $CHECKREPORT
		FOUND=1
	fi
fi

# If there are results, mail them to sysadmin

if [ "$ATTACK" -eq 1 ]; then
	cat $CHECKREPORT | $MAIL -s "$HOSTNAME $DATE ACTIVE SYSTEM ATTACK!" $SYSADMIN
elif [ "$FOUND" -eq 1 ]; then
	cat $CHECKREPORT | $MAIL -s "$HOSTNAME $DATE system check" $SYSADMIN
fi

# Clean Up
rm -f $CHECKFILE $CHECKOUTPUT $CHECKREPORT
rm -rf $REALTMPDIR
